        TITLE   Book psllX86.asm - Packed Shift Left Logical (X86)
        PAGE    53,160

;
; psllX86.asm - Packed Shift Left Logical
;
; This document contains file that you may use at your own risk in your own programs.
;
; This module is designed to accomodate X86 processors in the book.
;
;       NOTE: Needs masm 6.14 or later!
;
;       Created - 19 Dec 2001 - J.Leiterman
;
;       Modifications
;
;               
;
;       Tabs = 8



        .XLIST

        .387
        .686P
        .MMX
;       .K3D
        .model flat, C

        include x86.inc
        include KariType.inc
 
        .LIST



        .data

externdef       rrMaskBQ:qword



psllX86  SEGMENT USE32 PUBLIC 'CODE'



;_________________________________________________________________________________________________
;=================================================================================================
;       Packed Shift Left Logical 16x8bit by n:{0...7}
;
;void vmp_psllB( uint8 * const pbD, const uint8 * const pbA, uint count )
;{
;       uint32 msk;
;
;       count &= (8-1);                         // Clip count to data bit size -1
;
;       msk = rrMaskBD[count];
;       *(((uint32*)pbD)+0) = (*(((uint32*)pbA)+0) & msk) << count;
;       *(((uint32*)pbD)+1) = (*(((uint32*)pbA)+1) & msk) << count;
;       *(((uint32*)pbD)+2) = (*(((uint32*)pbA)+2) & msk) << count;
;       *(((uint32*)pbD)+3) = (*(((uint32*)pbA)+3) & msk) << count;
;}
;
;       NOTE: As X86 packed shifting only supports 16/32/64 bits we have to mask for 8bit!
;
;       *** MMX version *** 128bit (un)aligned

        public  vmp_psllBAsmMMX
vmp_psllBAsmMMX       proc    near pbD:ptr, pbA:ptr, count:DWORD
        .MMX    
ASSERT_PTR(pbD)
ASSERT_PTR(pbA)

        mov     ecx,dword ptr count             ; # of bits to shift
        mov     edx,pbD
        mov     eax,pbA
        and     ecx,8-1                         ; Clip count to 0...7 bits

        movq    mm0,[eax+0]                     ; Read Data
        movq    mm1,[eax+8]                             
        movd    mm2,ecx

        ; mm0=64bits mm1=64bits  mm2=# of bits to shift

        pand    mm0,rrMaskBQ[ecx*8]             ; Strip lower bits
        pand    mm1,rrMaskBQ[ecx*8]

        psllq   mm0,mm2                         ; val << count
        psllq   mm1,mm2

        movq    [edx+0],mm0                     ; Write Data
        movq    [edx+8],mm1

        ret
vmp_psllBAsmMMX endp




;_________________________________________________________________________________________________
;       Packed Shift Left Logical 16x8bit by 1
;
;void vmp_psll1B( uint8 * const pbD, const uint8 * const pbA )
;{
;       *(((uint32*)pbD)+0) = (*(((uint32*)pbA)+0) & 0x7f7f7f7f) << 1;
;       *(((uint32*)pbD)+1) = (*(((uint32*)pbA)+1) & 0x7f7f7f7f) << 1;
;       *(((uint32*)pbD)+2) = (*(((uint32*)pbA)+2) & 0x7f7f7f7f) << 1;
;       *(((uint32*)pbD)+3) = (*(((uint32*)pbA)+3) & 0x7f7f7f7f) << 1;
;}
;       NOTE: As X86 packed shifting only supports 16/32/64 bits we have to mask for 8bit!
;         Of course Adding 8bit # to itself is the same!
;
;       *** MMX version *** 128bit (un)aligned

        public  vmp_psll1BAsmMMX
vmp_psll1BAsmMMX       proc    near pbD:ptr, pbA:ptr, count:DWORD
        .MMX    
ASSERT_PTR(pbD)
ASSERT_PTR(pbA)

        mov     edx,pbD
        mov     eax,pbA

        movq    mm0,[eax+0]                     ; Read Data
        movq    mm1,[eax+8]

        ; mm0=64bits mm1=64bits
if 01   ; a+a = a << 1
        paddb   mm0,mm0
        paddb   mm1,mm1
else
        pand    mm0,rrMaskBQ[1*8]               ; Strip lower bit
        pand    mm1,rrMaskBQ[1*8]

        psllq   mm0,1                           ; val << 1
        psllq   mm1,1
endif

        movq    [edx+0],mm0                     ; Write Data
        movq    [edx+8],mm1
                
        ret
vmp_psll1BAsmMMX        endp



;_________________________________________________________________________________________________
;=================================================================================================
;       Packed Shift Left Logical 8x16bit by n:{0...15}
;
;void vmp_psllH( uint16 * const phD, const uint16 * const phA, uint count )
;{
;       uint32 msk;
;
;       count &= (16-1);                                // Clip count to data bit size -1
;
;       msk = rrMaskHD[count];
;       *(((uint32*)phD)+0) = (*(((uint32*)phA)+0) & msk) << count;
;       *(((uint32*)phD)+1) = (*(((uint32*)phA)+1) & msk) << count;
;       *(((uint32*)phD)+2) = (*(((uint32*)phA)+2) & msk) << count;
;       *(((uint32*)phD)+3) = (*(((uint32*)phA)+3) & msk) << count;
;}                                                                      
;       *** MMX version *** 128bit (un)aligned

        public  vmp_psllHAsmMMX
vmp_psllHAsmMMX       proc    near pwD:ptr, pwA:ptr, count:DWORD
        .MMX    
ASSERT_PTR(pwD)
ASSERT_PTR(pwA)

        mov     ecx,dword ptr count             ; # of bits to shift
        mov     edx,pwD
        mov     eax,pwA
        and     ecx,16-1                                ; Clip count to 0...7 bits

        movq    mm0,[eax+0]                     ; Read Data
        movq    mm1,[eax+8]                             
        movd    mm2,ecx

        ; mm0=64bits mm1=64bits  mm2=# of bits to shift

        psllw   mm0,mm2                         ; val << count (0...15)
        psllw   mm1,mm2

        movq    [edx+0],mm0                     ; Write Data
        movq    [edx+8],mm1

        ret
vmp_psllHAsmMMX endp



;_________________________________________________________________________________________________
;       Packed Shift Left Logical 8x16bit by 1
;
;void vmp_psll1H( uint16 * const phD, const uint16 * const phA )
;{
;       *(((uint32*)phD)+0) = (*(((uint32*)phA)+0) & 0x7fff7fff) << 1;
;       *(((uint32*)phD)+1) = (*(((uint32*)phA)+1) & 0x7fff7fff) << 1;
;       *(((uint32*)phD)+2) = (*(((uint32*)phA)+2) & 0x7fff7fff) << 1;
;       *(((uint32*)phD)+3) = (*(((uint32*)phA)+3) & 0x7fff7fff) << 1;
;}
;       *** MMX version *** 128bit (un)aligned

        public  vmp_psll1HAsmMMX
vmp_psll1HAsmMMX       proc    near phD:ptr, phA:ptr, count:DWORD
        .MMX    
ASSERT_PTR(phD)
ASSERT_PTR(phA)

        mov     edx,phD
        mov     eax,phA

        movq    mm0,[eax+0]                     ; Read Data
        movq    mm1,[eax+8]

        ; mm0=64bits mm1=64bits

        psllw   mm0,1                           ; val << 1
        psllw   mm1,1

        movq    [edx+0],mm0                     ; Write Data
        movq    [edx+8],mm1
                
        ret
vmp_psll1HAsmMMX        endp



;_________________________________________________________________________________________________
;=================================================================================================
;       Packed Shift Left Logical 4x32bit by n:{0...31}
;
;void vmp_psllW( uint32 * const pwD, const uint32 * const pwA, uint count )
;{
;       count &= (32-1);                                // Clip count to data bit size -1
;
;       *(pwD+0) = (*(pwA+0)) << count;
;       *(pwD+1) = (*(pwA+1)) << count;
;       *(pwD+2) = (*(pwA+2)) << count;
;       *(pwD+3) = (*(pwA+3)) << count;
;}                                                                      
;       *** MMX version *** 128bit (un)aligned

        public  vmp_psllWAsmMMX
vmp_psllWAsmMMX       proc    near pwD:ptr, pwA:ptr, count:DWORD
        .MMX    
ASSERT_PTR(pwD)
ASSERT_PTR(pwA)

        mov     ecx,dword ptr count             ; # of bits to shift
        mov     edx,pwD
        mov     eax,pwA
        and     ecx,32-1                                ; Clip count to 0...7 bits

        movq    mm0,[eax+0]                     ; Read Data
        movq    mm1,[eax+8]                             
        movd    mm2,ecx

        ; mm0=64bits mm1=64bits  mm2=# of bits to shift

        pslld   mm0,mm2                         ; val << count (0...15)
        pslld   mm1,mm2

        movq    [edx+0],mm0                     ; Write Data
        movq    [edx+8],mm1

        ret
vmp_psllWAsmMMX endp



;_________________________________________________________________________________________________
;       Packed Shift Left Logical 4x32bit by 1
;
;void vmp_psll1W( uint32 * const pwD, const uint32 * const pwA )
;{
;       *(pwD+0) = *(pwA+0) << 1;
;       *(pwD+1) = *(pwA+1) << 1;
;       *(pwD+2) = *(pwA+2) << 1;
;       *(pwD+3) = *(pwA+3) << 1;
;}
;       *** MMX version *** 128bit (un)aligned

        public  vmp_psll1WAsmMMX
vmp_psll1WAsmMMX       proc    near pwD:ptr, pwA:ptr, count:DWORD
        .MMX    
ASSERT_PTR(pwD)
ASSERT_PTR(pwA)

        mov     edx,pwD
        mov     eax,pwA

        movq    mm0,[eax+0]                     ; Read Data
        movq    mm1,[eax+8]

        ; mm0=64bits mm1=64bits

        pslld   mm0,1                           ; val << 1
        pslld   mm1,1

        movq    [edx+0],mm0                     ; Write Data
        movq    [edx+8],mm1
                
        ret
vmp_psll1WAsmMMX        endp



psllX86  ends
        end




