        TITLE   FmdX86.asm - 80(x)86 Math Multiplication & Division
        PAGE    53,160

;  FmdX86.asm - (Scalar) Math Multiplication & Division
;
;  This document contains file that you may use at your own risk in your own programs.
;
;  This module is designed to accomodate the book.
;
;               This module is designed for fast 3D math
;
;  Created - 3 Oct 2001 - James Leiterman
;
;  Last Modification
;
;       Tabs = 8



        .XLIST

;;;        .387
;;;     .586P   .586    .686   .686P
;;;      .K3D    .XMM    .MMX

        .686
        .XMM

        .model flat, C


        include KariType.inc
        include vmp3DX86M.inc
 
        .LIST


        .data
        align   16


;------------------------------------------------------------------------------------------------


FmdX86  SEGMENT USE32 PUBLIC 'CODE'

;-------------------------------------------------------------------------------------------------




;_________________________________________________________________________________________________
;=================================================================================================
;       (float) Multiplication
;
;void vmp_FMul( float * const pfD, float fA, float fB )
;{
;       *pfD = A * B;
;}

;_________________________________________________________________________________________________

; *** 3DNow version ***
;

        align   16
        public  vmp_FMulAsm3DNow
vmp_FMulAsm3DNow       proc    near , pfD:ptr, fA:REAL4, fB:REAL4
        .K3D
;;;        femms

ASSERT_PTR(pfD)

        movd    mm0,fA                          ;       {0    A}
        movd    mm1,fB                          ;       {0    B}

        mov     eax,pfD                         ; Float Destination Pointer

        pfmul    mm0,mm1                                ;       {0*0 A*B}

        movd    [eax],mm0                       ;      A*B

;;;        femms
        ret
vmp_FMulAsm3DNow endp


;_________________________________________________________________________________________________



; *** SSE version ***
;
        align   16
        public  vmp_FMulAsmSSE
vmp_FMulAsmSSE       proc    near , pfD:ptr, fA:REAL4, fB:REAL4
        .XMM
;;;        emms

ASSERT_PTR(pfD)

        movss   xmm0,fA                         ;A       {0 0 0 A}
        movss   xmm1,fB                         ;B       {0 0 0 B}

        mov     eax,pfD                         ; Float Destination Pointer
        mulss   xmm0,xmm1                       ;        {0 0 0 A*B}

        movss   [eax],xmm0                      ;        A*B    
        
;;;        emms
        ret
vmp_FMulAsmSSE endp



;_________________________________________________________________________________________________
;=================================================================================================
;       (float) Divide
;
;void vmp_FDiv( float * const pfD, float fA, float fB )
;{
;       *pfD = A / B;
;}

;_________________________________________________________________________________________________

; *** 3DNow version ***    (24bits precision)
;

        align   16
        public  vmp_FDivAsm3DNow
vmp_FDivAsm3DNow       proc    near , pfD:ptr, fA:REAL4, fB:REAL4
        .K3D
;;;        femms

ASSERT_PTR(pfD)

        movd    mm3,fB                          ;       {0    fB}

         ;      Calculate reciprocal of source B
         ;      pfrcp=14bit accuracy
        pfrcp    mm1,mm3                                ;X0=f(B)=1/B  {1/fB   1/fB}

        mov     eax,pfD                         ; Float Destination Pointer

         ;      refine Newton-Raphson reciprocal square approximation
         ;      Calculate 1/sqrt() accurate to 24bits
        pfrcpit1 mm3,mm1                                ;X1=g1(B,X0)   {1st step}

        movd    mm0,fA                          ;       {0    fA}

        pfrcpit2 mm3,mm1                                ;X2=g2(X1,X0)  {2nd step}
        pfmul   mm0,mm3         ; 24bits          ; =a*X2        {#   A*(1/B)}
        
        movd    [eax],mm0                       ;        A/B

;;;        femms
        ret
vmp_FDivAsm3DNow endp


;_________________________________________________________________________________________________
;       (float) Divide  ***FAST 14bits precision***
;

; *** 3DNow version ***
;

        align   16
        public  vmp_FDivFastAsm3DNow
vmp_FDivFastAsm3DNow       proc    near , pfD:ptr, fA:REAL4, fB:REAL4
        .K3D
;;;        femms

ASSERT_PTR(pfD)

        movd    mm2,fB                          ;       {0    B}
        movd    mm0,fA                          ;       {0    A}
        mov     eax,pfD                         ; Float Destination Pointer

         ;      Calculate reciprocal of source B
         ;      pfrcp=14bit accuracy
        pfrcp    mm1,mm2                                ;X0=f(B)=1/B  {1/B    1/B}
        pfmul   mm0,mm1         ; 24bits          ; =a*X2       {0   A*(1/B)}
        
        movd    [eax],mm0                       ;        A/B

;;;        femms
        ret
vmp_FDivFastAsm3DNow endp

;_________________________________________________________________________________________________



; *** SSE version *** non aligned
;
        align   16
        public  vmp_FDivAsmSSE
vmp_FDivAsmSSE       proc    near , pfD:ptr, fA:REAL4, fB:REAL4
        .XMM
;;;        emms

ASSERT_PTR(pfD)

        movss   xmm1,fB                         ;B       {0 0 0 B}
        movss   xmm0,fA                         ;A       {0 0 0 A}

        mov     eax,pfD                         ; Float Destination Pointer
        divss   xmm0,xmm1                       ;        {0 0 0 A/B}

        movss   [eax],xmm0                      ;        A/B    
        
;;;        emms
        ret
vmp_FDivAsmSSE endp


;_________________________________________________________________________________________________



; *** SSE version *** non aligned
;
        align   16
        public  vmp_FDivFastAsmSSE
vmp_FDivFastAsmSSE       proc    near , pfD:ptr, fA:REAL4, fB:REAL4
        .XMM
;;;        emms

ASSERT_PTR(pfD)

        movss   xmm1,fB                         ;B       {0 0 0 B}
        movss   xmm0,fA                         ;A       {0 0 0 A}

        mov     eax,pfD                         ; Float Destination Pointer

        rcpss   xmm1,xmm1                       ;        {0 0 0 1/s}
        mulss   xmm0,xmm1                       ;        {0 0 0 A*1/s}

        movss   [eax],xmm0                      ;        A/B    
        
;;;        emms
        ret
vmp_FDivFastAsmSSE endp





FmdX86  ends
        end

