
//
//  QVmd3DX86.cpp - (Quad) 3D Vector Multiplication & Division {Altivec}
//
//  This document contains file that you may use at your own risk in your own programs.
//
//  This module is designed to work with the exercises for this book.
//
//  Created - 3 Oct 2001 - James Leiterman
//
//  Last Modification
//
//  stab=4
//


#include "KariType.h"                                   // Master Definitions

    //  3rd Party Libraries
    
#include    <assert.h>
#include    <iostream.h>

    //  Book Development Library Modules

#include    "CpuAsm.h"                                  // CPU
#include    "vmp3D.h"
#include    "vmp3DEx.h"
#include    "QVmd3D.h"                                  // 3D Math
#include    "QVmd3DEx.h"



//
//  NOTE: This function sets up the function vectors so the best function
//      is attached to the vector for the processor running.
//

void QVmd3DSetup( const CpuInfo * const pcinfo )
{
    ASSERT_PTR4(pcinfo);
    
    if ( CPUBITS_SSE & pcinfo->nBits )
      {
        vmp_QVecMul =           vmp_QVecMulAsmSSE;
        vmp_QVecMul0 =          vmp_QVecMulAsmSSE0;
        vmp_QVecMAdd =          vmp_QVecMAddAsmSSE;
        vmp_QVecMAdd0 =         vmp_QVecMAddAsmSSE0;
        vmp_QVecMSub =          vmp_QVecMSubAsmSSE;
        vmp_QVecMSub0 =         vmp_QVecMSubAsmSSE0;
        vmp_QVecScale =         vmp_QVecScaleAsmSSE;
        vmp_QVecScale0 =        vmp_QVecScaleAsmSSE0;
        vmp_QVecDiv =           vmp_QVecDivAsmSSE;
        vmp_QVecDiv0 =          vmp_QVecDivAsmSSE0;
        vmp_QVecDivFast =       vmp_QVecDivFastAsmSSE;
        vmp_QVecDivFast0 =      vmp_QVecDivFastAsmSSE0;
        vmp_QDotProduct =       vmp_QDotProductAsmSSE;
        vmp_QDotProduct0 =      vmp_QDotProductAsmSSE0;
        vmp_QCrossProduct =     vmp_QCrossProductAsmSSE;
        vmp_QCrossProduct0 =    vmp_QCrossProductAsmSSE0;
      }

    else if ( CPUBITS_3DNOW & pcinfo->nBits )
      {
        vmp_QVecMul =           vmp_QVecMulAsm3DNow;
        vmp_QVecMul0 =          vmp_QVecMulAsm3DNow;
        vmp_QVecMAdd =          vmp_QVecMAddAsm3DNow;
        vmp_QVecMAdd0 =         vmp_QVecMAddAsm3DNow;
        vmp_QVecMSub =          vmp_QVecMSubAsm3DNow;
        vmp_QVecMSub0 =         vmp_QVecMSubAsm3DNow;
        vmp_QVecScale =         vmp_QVecScaleAsm3DNow;
        vmp_QVecScale0 =        vmp_QVecScaleAsm3DNow;
        vmp_QVecDiv =           vmp_QVecDivAsm3DNow;
        vmp_QVecDiv0 =          vmp_QVecDivAsm3DNow;
        vmp_QVecDivFast =       vmp_QVecDivFastAsm3DNow;    //***FAST***
        vmp_QVecDivFast0 =      vmp_QVecDivFastAsm3DNow;    //***FAST***
        vmp_QDotProduct =       vmp_QDotProductAsm3DNow;
        vmp_QDotProduct0 =      vmp_QDotProductAsm3DNow;
        vmp_QCrossProduct =     vmp_QCrossProductAsm3DNow;
        vmp_QCrossProduct0 =    vmp_QCrossProductAsm3DNow;
      }

    else
      {
        vmp_QVecMul =           vmp_QVecMulGeneric;
        vmp_QVecMul0 =          vmp_QVecMulGeneric;
        vmp_QVecMAdd =          vmp_QVecMAddGeneric;
        vmp_QVecMAdd0 =         vmp_QVecMAddGeneric;
        vmp_QVecMSub =          vmp_QVecMSubGeneric;
        vmp_QVecMSub0 =         vmp_QVecMSubGeneric;
        vmp_QVecScale =         vmp_QVecScaleGeneric;
        vmp_QVecScale0 =        vmp_QVecScaleGeneric;
        vmp_QVecDiv =           vmp_QVecDivGeneric;
        vmp_QVecDiv0 =          vmp_QVecDivGeneric;
        vmp_QVecDivFast =       vmp_QVecDivGeneric;
        vmp_QVecDivFast0 =      vmp_QVecDivGeneric;
        vmp_QDotProduct =       vmp_QDotProductGeneric;
        vmp_QDotProduct0 =      vmp_QDotProductGeneric;
        vmp_QCrossProduct =     vmp_QCrossProductGeneric;
        vmp_QCrossProduct0 =    vmp_QCrossProductGeneric;
      }
}



