
//
//  QVsf3DX86.cpp - 3D Vector Special (Quad) Function
//
//  This document contains file that you may use at your own risk in your own programs.
//
//  This module is designed to work with the exercises for this book.
//
//  NOTE: Because these a 3 float Vectors the 4th float has to be handled as 
//      a special case thus slowing the code down. This is the same as an AoS 
//      orientation. Instead a SoA should be used in conjunction with the quad
//      set of functions.
//
//  Created - 3 Oct 2001 - James Leiterman
//
//  Last Modification
//
//  stab=4
//



#include "KariType.h"                                   // Master Definitions

    //  3rd Party Libraries
    
#include    <assert.h>
#include    <iostream.h>
#include    <math.h>

    //  Book Development Library Modules

#include    "CpuAsm.h"                                  // CPU
#include    "vmp3D.h"
#include    "vmp3DEx.h"

#include    "QVsf3D.h"                                  // 3D Math
#include    "QVsf3DEx.h"




//______________________________________________________
//
//  Setup Vec3D module
//
//  NOTE: This function sets up the function vectors so the best function
//      is attached to the vector for the processor running.
//

void QVsf3DSetup( const CpuInfo * const pcinfo )
{
    ASSERT_PTR4(pcinfo);

    if ( CPUBITS_SSE & pcinfo->nBits )
      {
        vmp_QVecSqrt =              vmp_QVecSqrtAsmSSE;
        vmp_QVecSqrt0 =             vmp_QVecSqrtAsmSSE0;
        vmp_QVecSqrtFast =          vmp_QVecSqrtFastAsmSSE;         // ***FAST***
        vmp_QVecSqrtFast0 =         vmp_QVecSqrtFastAsmSSE0;        // ***FAST***

        vmp_QVecMin =               vmp_QVecMinAsmSSE;
        vmp_QVecMin0 =              vmp_QVecMinAsmSSE0;
        vmp_QVecMax =               vmp_QVecMaxAsmSSE;
        vmp_QVecMax0 =              vmp_QVecMaxAsmSSE0;

        vmp_QVecMagnitude =         vmp_QVecMagnitudeAsmSSE;
        vmp_QVecMagnitude0 =        vmp_QVecMagnitudeAsmSSE0;
        vmp_QVecMagnitudeFast =     vmp_QVecMagnitudeFastAsmSSE;    // ***FAST***
        vmp_QVecMagnitudeFast0 =    vmp_QVecMagnitudeFastAsmSSE0;   // ***FAST***

        vmp_QVecNormalize =         vmp_QVecNormalizeAsmSSE;
        vmp_QVecNormalize0 =        vmp_QVecNormalizeAsmSSE0;
        vmp_QVecNormalizeFast =     vmp_QVecNormalizeFastAsmSSE;    // ***FAST***
        vmp_QVecNormalizeFast0 =    vmp_QVecNormalizeFastAsmSSE0;   // ***FAST***
      }

    else if ( CPUBITS_3DNOW & pcinfo->nBits )
      {
        vmp_QVecSqrt =              vmp_QVecSqrtAsm3DNow;
        vmp_QVecSqrt0 =             vmp_QVecSqrtAsm3DNow;
        vmp_QVecSqrtFast =          vmp_QVecSqrtFastAsm3DNow;       // ***FAST***
        vmp_QVecSqrtFast0 =         vmp_QVecSqrtFastAsm3DNow;       // ***FAST***

        vmp_QVecMin =               vmp_QVecMinAsm3DNow;
        vmp_QVecMin0 =              vmp_QVecMinAsm3DNow;
        vmp_QVecMax =               vmp_QVecMaxAsm3DNow;
        vmp_QVecMax0 =              vmp_QVecMaxAsm3DNow;

        vmp_QVecMagnitude =         vmp_QVecMagnitudeAsm3DNow;
        vmp_QVecMagnitude0 =        vmp_QVecMagnitudeAsm3DNow;
        vmp_QVecMagnitudeFast =     vmp_QVecMagnitudeFastAsm3DNow;  // ***FAST***
        vmp_QVecMagnitudeFast0 =    vmp_QVecMagnitudeFastAsm3DNow;  // ***FAST***

        vmp_QVecNormalize =         vmp_QVecNormalizeAsm3DNow;
        vmp_QVecNormalize0 =        vmp_QVecNormalizeAsm3DNow;
        vmp_QVecNormalizeFast =     vmp_QVecNormalizeFastAsm3DNow;  // ***FAST***
        vmp_QVecNormalizeFast0 =    vmp_QVecNormalizeFastAsm3DNow;  // ***FAST***
      }

    else
      {
        vmp_QVecSqrt =              vmp_QVecSqrtGeneric;
        vmp_QVecSqrt0 =             vmp_QVecSqrtGeneric;
        vmp_QVecSqrtFast =          vmp_QVecSqrtGeneric;
        vmp_QVecSqrtFast0 =         vmp_QVecSqrtGeneric;

        vmp_QVecMin =               vmp_QVecMinGeneric;
        vmp_QVecMin0 =              vmp_QVecMinGeneric;
        vmp_QVecMax =               vmp_QVecMaxGeneric;
        vmp_QVecMax0 =              vmp_QVecMaxGeneric;

        vmp_QVecMagnitude =         vmp_QVecMagnitudeGeneric;
        vmp_QVecMagnitude0 =        vmp_QVecMagnitudeGeneric;
        vmp_QVecMagnitudeFast =     vmp_QVecMagnitudeGeneric;
        vmp_QVecMagnitudeFast0 =    vmp_QVecMagnitudeGeneric;

        vmp_QVecNormalize =         vmp_QVecNormalizeGeneric;
        vmp_QVecNormalize0 =        vmp_QVecNormalizeGeneric;
        vmp_QVecNormalizeFast =     vmp_QVecNormalizeGeneric;
        vmp_QVecNormalizeFast0 =    vmp_QVecNormalizeGeneric;
      }
}





