        TITLE   Matrix3DX86M.asm - 80(x)86 3D Math Matrix Function
        PAGE    53,160

;  Matrix3DX86.asm - 3D Matrix Math
;
;  This document contains file that you may use at your own risk in your own programs.
;
;  This module is designed to accomodate the book and are not optimized only functional.
;
;               This module is designed for fast 3D math
;
;       The Matrices in this book are 4x4 of which its elements are arranged linearly
;       within memory. This file uses different references for addressing vector rows 
;       and individual elements. Please note that the first float in memory occurs as
;       the upper left corner of the matrix such as in the tables on the left, but in
;       terms of 128bit Vectors and little-endian orientation, the first float is more
;       logically pictured as show in the image on the right. 
;
;         Matrix            Structure Addressing                Little-Endian Mapping
;       [0 1 2 3]    [_m00 _m01 _m02 _m03]                  [a3 a2 a1 a0]
;       [4 5 6 7]    [_m10 _m11 _m12 _m13]                  [a7 a6 a5 a4]     
;       [8 9 A B]    [_m20 _m21 _m22 _m23]                  [aB aA a9 a8]
;       [C D E F]    [_m30 _m31 _m32 _m33]                  [aF aE aD aC]
;
;       Thus quite often two tables of results will be shown in the comments area. The
;       matrix on the left, and the little-endian mapping on the right such as the
;       following matrix Identity.
;
;                                     ; 1 0 0 0        [0 0 0 1]
;                                     ; 0 1 0 0        [0 0 1 0]
;                                     ; 0 0 1 0        [0 1 0 0]
;                                     ; 0 0 0 1        [1 0 0 0]
;
;  Created - 3 Oct 2001 - James Leiterman
;
;  Last Modification
;
;       Tabs = 8



        .XLIST

        .686
        .XMM

        .model flat, C

 
        include KariType.inc
        include vmp3DX86M.inc

        .LIST


OFLOW_TEST      equ     1       ; Overflow testing & fixup




        .data
        align   16

;       Used for Identity Matrix
        REAL4   0.0, 0.0, 0.0, 0.0      ; Keep these two (0000 & 0001) together!
vec1000 REAL4   1.0, 0.0, 0.0, 0.0






;------------------------------------------------------------------------------------------------


Matrix3DX86     SEGMENT USE32 PUBLIC 'CODE'

;-------------------------------------------------------------------------------------------------



;_________________________________________________________________________________________________
;=================================================================================================
;       Matrix - Copy  mD = mA
;
;void vmp_MatrixCopy( vmp3DMatrix dMx, const vmp3DMatrix aMx )
;{
;       dMx[0][0]=aMx[0][0];  dMx[0][1]=aMx[0][1];  dMx[0][2]=aMx[0][2];  dMx[0][3]=aMx[0][3];
;       dMx[1][0]=aMx[1][0];  dMx[1][1]=aMx[1][1];  dMx[1][2]=aMx[1][2];  dMx[1][3]=aMx[1][3];
;       dMx[2][0]=aMx[2][0];  dMx[2][1]=aMx[2][1];  dMx[2][2]=aMx[2][2];  dMx[2][3]=aMx[2][3];
;       dMx[3][0]=aMx[3][0];  dMx[3][1]=aMx[3][1];  dMx[3][2]=aMx[3][2];  dMx[3][3]=aMx[3][3];
;}
;_________________________________________________________________________________________________

        public  vmp_MatrixCopyAsm
vmp_MatrixCopyAsm       proc    near    ;;; , dMx:ptr, aMx:ptr

ASSERT_PTR4( DWORD PTR [esp+8] )                ; aMx
ASSERT_PTR4( DWORD PTR [esp+4] )                ; dMx

        mov     eax,[esp+8]             ; aMx
        mov     edx,[esp+4]             ; dMx

        push    ebx

        mov     ecx,[eax+0]
        mov     ebx,[eax+4]

        i = 0
REPT    16/2 -1
        mov     [edx+0+i],ecx
        mov     ecx,[eax+8+i]
        mov     [edx+4+i],ebx
        mov     ebx,[eax+12+i]
        i = i + 8
ENDM

        mov     [edx+0+i],ecx
        mov     [edx+4+i],ebx

        pop     ebx
        ret
vmp_MatrixCopyAsm       endp


;       SSE Version

        public  vmp_MatrixCopyAsmSSE
vmp_MatrixCopyAsmSSE    proc    near    ;;; , dMx:ptr, aMx:ptr
        .XMM

ASSERT_PTR16( DWORD PTR [esp+8] )               ; aMx
ASSERT_PTR16( DWORD PTR [esp+4] )               ; dMx

        mov     eax,[esp+8]             ; aMx
        mov     edx,[esp+4]             ; dMx

        movaps  xmm0,[eax+0]            ; [a3 a2 a1 a0]
        movaps  xmm1,[eax+16]           ; [a7 a6 a5 a4]
        movaps  xmm2,[eax+32]           ; [aB aA a9 a8]
        movaps  xmm3,[eax+48]           ; [aF aE aD aC]

        movaps  [edx+0],xmm0            ; [d3 d2 d1 d0]
        movaps  [edx+16],xmm1           ; [d7 d6 d5 d4]
        movaps  [edx+32],xmm2           ; [dB dA d9 d8]
        movaps  [edx+48],xmm3           ; [dF dE dD dC]

        ret
vmp_MatrixCopyAsmSSE    endp



;_________________________________________________________________________________________________
;=================================================================================================
;       Matrix Set Identity
;
;void vmp_MatrixSetIdentity( vmp3DMatrix * const matrix );
;
;       Builds a 4x4 identity matrix
;
;       1 0 0 0
;       0 1 0 0
;       0 0 1 0
;       0 0 0 1
;_________________________________________________________________________________________________

        public  vmp_MatrixSetIdentityAsm
vmp_MatrixSetIdentityAsm        proc    near

ASSERT_PTR4( DWORD PTR [esp+4] )

        mov     eax,[esp+4]
        mov     ecx,FLOAT1              ; 1.0f
        mov     edx,FLOAT0              ; 0.0f

        mov     (vmp3DMatrix PTR [eax])._m00,ecx        ;[a0] = 1.0
        mov     (vmp3DMatrix PTR [eax])._m01,edx        ;[a1] = 0.0
        mov     (vmp3DMatrix PTR [eax])._m02,edx        ;[a2]
        mov     (vmp3DMatrix PTR [eax])._m03,edx        ;[a3]

        mov     (vmp3DMatrix PTR [eax])._m10,edx        ;[a4]
        mov     (vmp3DMatrix PTR [eax])._m11,ecx        ;[a5] = 1.0
        mov     (vmp3DMatrix PTR [eax])._m12,edx        ;[a6]
        mov     (vmp3DMatrix PTR [eax])._m13,edx        ;[a7]

        mov     (vmp3DMatrix PTR [eax])._m20,edx        ;[a8]
        mov     (vmp3DMatrix PTR [eax])._m21,edx        ;[a9]
        mov     (vmp3DMatrix PTR [eax])._m22,ecx        ;[aA] = 1.0
        mov     (vmp3DMatrix PTR [eax])._m23,edx        ;[aB]

        mov     (vmp3DMatrix PTR [eax])._m30,edx        ;[aC]
        mov     (vmp3DMatrix PTR [eax])._m31,edx        ;[aD]
        mov     (vmp3DMatrix PTR [eax])._m32,edx        ;[aE]
        mov     (vmp3DMatrix PTR [eax])._m33,ecx        ;[aF] = 1.0
        ret
vmp_MatrixSetIdentityAsm        endp



;       MMX Version (Only use for 3DNow! Not worth it otherwise due to EMMS!

        public  vmp_MatrixSetIdentityAsmMMX
vmp_MatrixSetIdentityAsmMMX     proc    near
        .MMX

ASSERT_PTR4( DWORD PTR [esp+4] )

        mov     eax,[esp+4]

        movq    mm2,qword ptr vec1000-4 ; 1.0 0.0    1 0 X X
        pxor    mm0,mm0                 ; 0.0 0.0    0 0 X X
        movd    mm1,vec1000             ; 0.0 1.0    0 1 X X

        movq    (qword ptr [eax+0 ]),mm1        ;00,01 = 1.0 0.0
        movq    (qword ptr [eax+08]),mm0        ;02,03 = 0.0 0.0

        movq    (qword ptr [eax+16]),mm2        ;10,11 = 0.0 1.0
        movq    (qword ptr [eax+24]),mm0        ;12,13 =         0.0 0.0
           
        movq    (qword ptr [eax+32]),mm0        ;20,21 = 0.0 0.0
        movq    (qword ptr [eax+40]),mm1        ;22,23 =         1.0 0.0

        movq    (qword ptr [eax+48]),mm0        ;30,31 = 0.0 0.0
        movq    (qword ptr [eax+56]),mm2        ;32,33 =         0.0 1.0

        ret
vmp_MatrixSetIdentityAsmMMX     endp



;       SSE Version - Aligned

        public  vmp_MatrixSetIdentityAsmSSE
vmp_MatrixSetIdentityAsmSSE     proc    near
        .XMM

ASSERT_PTR16( DWORD PTR [esp+4] )
                                        ;      x y z w       w z y x
        movaps  xmm0,oword ptr vec1000  ; aMx:[1 0 0 0]    H[0 0 0 1]L
        mov     eax,[esp+4]

        movaps  xmm1,xmm0
        movaps  (oword ptr [eax+0 ]),xmm0
        shufps  xmm0,xmm0,11110011b             ; 0 1 0 0  [0 0 1 0]
        shufps  xmm1,xmm1,11001111b             ; 0 0 1 0  [0 1 0 0]
        movaps  (oword ptr [eax+16]),xmm0
        shufps  xmm0,xmm0,01111111b             ; 0 0 0 1  [1 0 0 0]
        movaps  (oword ptr [eax+32]),xmm1
        movaps  (oword ptr [eax+48]),xmm0

        ret
vmp_MatrixSetIdentityAsmSSE     endp



;_________________________________________________________________________________________________
;=================================================================================================
;       Matrix - Transpose  mD = (mA)'
;
;void vmp_MatrixCopy( vmp3DMatrix dMx, const vmp3DMatrix aMx )
;{
;       dMx[0][0]=aMx[0][0];  dMx[0][1]=aMx[1][0];  dMx[0][2]=aMx[2][0];  dMx[3][0]=aMx[0][3];
;       dMx[1][0]=aMx[0][1];  dMx[1][1]=aMx[1][1];  dMx[1][2]=aMx[2][1];  dMx[3][1]=aMx[1][3];
;       dMx[2][0]=aMx[0][2];  dMx[2][1]=aMx[1][2];  dMx[2][2]=aMx[2][2];  dMx[3][2]=aMx[2][3];
;       dMx[3][0]=aMx[0][3];  dMx[3][1]=aMx[1][3];  dMx[3][2]=aMx[2][3];  dMx[3][3]=aMx[3][3];
;}
;_________________________________________________________________________________________________

        public  vmp_MatrixTransposeAsm
vmp_MatrixTransposeAsm  proc    near    ;;; , dMx:ptr, aMx:ptr

ASSERT_PTR4( DWORD PTR [esp+8] )                ; aMx
ASSERT_PTR4( DWORD PTR [esp+4] )                ; dMx

        mov     eax,[esp+8]             ; aMx
        mov     edx,[esp+4]             ; dMx

        push    ebx

                ; [0 1 2 3]    [a3 a2 a1 a0]
                ; [4 5 6 7]    [a7 a6 a5 a4]
                ; [8 9 A B]    [aB aA a9 a8]
                ; [C D E F]    [aF aE aD aC]

        mov     ecx,[eax+(0*4)]
        mov     ebx,[eax+(5*4)]
        mov     [edx+(0*4)],ecx
        mov     [edx+(5*4)],ebx

        mov     ecx,[eax+(1*4)]
        mov     ebx,[eax+(4*4)]
        mov     [edx+(4*4)],ecx
        mov     [edx+(1*4)],ebx

        mov     ecx,[eax+(2*4)]
        mov     ebx,[eax+(8*4)]
        mov     [edx+(8*4)],ecx
        mov     [edx+(2*4)],ebx

        mov     ecx,[eax+(3*4)]
        mov     ebx,[eax+(12*4)]
        mov     [edx+(12*4)],ecx
        mov     [edx+(3*4)],ebx

        mov     ecx,[eax+(6*4)]
        mov     ebx,[eax+(9*4)]
        mov     [edx+(9*4)],ecx
        mov     [edx+(6*4)],ebx

        mov     ecx,[eax+(7*4)]
        mov     ebx,[eax+(13*4)]
        mov     [edx+(13*4)],ecx
        mov     [edx+(7*4)],ebx

        mov     ecx,[eax+(11*4)]
        mov     ebx,[eax+(14*4)]
        mov     [edx+(14*4)],ecx
        mov     [edx+(11*4)],ebx

        mov     ecx,[eax+(10*4)]
        mov     ebx,[eax+(15*4)]
        mov     [edx+(10*4)],ecx
        mov     [edx+(15*4)],ebx

        pop     ebx
        ret
vmp_MatrixTransposeAsm  endp



;       SSE Version

        public  vmp_MatrixTransposeAsmSSE
vmp_MatrixTransposeAsmSSE       proc    near    ;;; , dMx:ptr, aMx:ptr
        .XMM

ASSERT_PTR16( DWORD PTR [esp+8] )               ; aMx
ASSERT_PTR16( DWORD PTR [esp+4] )               ; dMx

        mov     eax,[esp+8]             ; aMx
        mov     edx,[esp+4]             ; dMx

        ;       Transpose from AoS {XYZW}[4] to SoA X[4],Y[4],Z[4],W[4]

        movaps  xmm0,[eax+0]            ; [0 1 2 3]    [a3 a2 a1 a0]
        movaps  xmm1,[eax+16]           ; [4 5 6 7]    [a7 a6 a5 a4]
        movaps  xmm4,[eax+32]           ; [8 9 A B]    [aB aA a9 a8]
        movaps  xmm3,[eax+48]           ; [C D E F]    [aF aE aD aC]

        movaps  xmm2,xmm0               ; [0 1 2 3]    [a3 a2 a1 a0]
        movaps  xmm5,xmm1               ; [4 5 6 7]    [a7 a6 a5 a4]

                ;       Get elements grouped into proper quad corner
        shufps  xmm0,xmm3,01000100b     ; [0 1 C D]    [aD aC a1 a0]
        shufps  xmm1,xmm4,01000100b     ; [4 5 8 9]    [a9 a8 a5 a4]
        shufps  xmm2,xmm4,11101110b     ; [2 3 A B]    [aB aA a3 a2]
        shufps  xmm5,xmm3,11101110b     ; [6 7 E F]    [aF aE a7 a6]

                ;       Get elements onto correct row
        movaps  xmm4,xmm0               ; [0 1 C D]    [aD aC a1 a0]
        movaps  xmm3,xmm2               ; [2 3 A B]    [aB aA a3 a2]
                ;       Upper
        shufps  xmm0,xmm1,10001000b     ; [0 C 4 8]    [a8 a4 aC a0]
        shufps  xmm1,xmm4,11011101b     ; [5 9 1 D]    [aD a1 a9 a5]
                ;       Lower
        shufps  xmm2,xmm5,10001000b     ; [2 A 6 E]    [aE a6 aA a2]
        shufps  xmm3,xmm5,11011101b     ; [3 B 7 F]    [aF a7 aB a3]
        
                ;       Horizontal Shuffle each row
        shufps  xmm0,xmm0,01111000b     ; [0 4 8 C]    [aC a8 a4 a0]
        shufps  xmm1,xmm1,11010010b     ; [1 5 9 D]    [aD a9 a5 a1]
        shufps  xmm2,xmm2,11011000b     ; [2 6 A E]    [aE aA a6 a2]
        shufps  xmm3,xmm3,11011000b     ; [3 7 B F]    [aF aB a7 a3]

        movaps  [edx+0],xmm0            ; [0 4 8 C]    [aC a8 a4 a0]
        movaps  [edx+16],xmm1           ; [1 5 9 D]    [aD a9 a5 a1]
        movaps  [edx+32],xmm2           ; [2 6 A E]    [aE aA a6 a2]
        movaps  [edx+48],xmm3           ; [3 7 B F]    [aF aB a7 a3]

        ret
vmp_MatrixTransposeAsmSSE       endp



;_________________________________________________________________________________________________
;=================================================================================================
;       Matrix - Set Translate
;
;        | 1  0  0  0 |
;        | 0  1  0  0 |
;        | 0  0  1  0 |
;        | Tx Ty Tz 1 |
;
;void vmp_MatrixSetTranslate( vmp3DMatrix Mx, float fTx, float fTy, float fTz )
;{
;                     Mx[0][1]=     Mx[0][2]=     Mx[0][3] = 
;       Mx[1][0]=                   Mx[1][2]=     Mx[1][3] = 
;       Mx[2][0]=     Mx[2][1]=                   Mx[2][3] = 0.0f;
;       Mx[0][0]=     Mx[1][1]=     Mx[2][2]=     Mx[3][3] = 1.0f;
;
;    Mx[3][0]=fTx; Mx[3][1]=fTy; Mx[3][2]=fTz;
;}
;_________________________________________________________________________________________________

        public  vmp_MatrixSetTranslateAsm
vmp_MatrixSetTranslateAsm       proc    near

ASSERT_PTR4( DWORD PTR [esp+4] )                ; dMx

        mov     eax,[esp+4]             ; dMx

        mov     ecx,FLOAT1              ; 1.0f
        mov     edx,FLOAT0              ; 0.0f

        mov     (vmp3DMatrix PTR [eax])._m00,ecx        ;11 = 1.0
        mov     (vmp3DMatrix PTR [eax])._m01,edx        ;12 = 0.0
        mov     (vmp3DMatrix PTR [eax])._m02,edx        ;13
        mov     (vmp3DMatrix PTR [eax])._m03,edx        ;14

        mov     (vmp3DMatrix PTR [eax])._m10,edx        ;21
        mov     (vmp3DMatrix PTR [eax])._m11,ecx        ;22 = 1.0
        mov     (vmp3DMatrix PTR [eax])._m12,edx        ;23
        mov     (vmp3DMatrix PTR [eax])._m13,edx        ;24

        mov     (vmp3DMatrix PTR [eax])._m20,edx        ;31
        mov     (vmp3DMatrix PTR [eax])._m21,edx        ;32
        mov     (vmp3DMatrix PTR [eax])._m22,ecx        ;33 = 1.0
        mov     (vmp3DMatrix PTR [eax])._m23,edx        ;34

        mov     (vmp3DMatrix PTR [eax])._m33,ecx        ;44 = 1.0

        mov     ecx,[esp+8]             ; fTx
        mov     edx,[esp+12]            ; fTy
        mov     (vmp3DMatrix PTR [eax])._m30,ecx        ;41
        mov     ecx,[esp+16]            ; fTz
        mov     (vmp3DMatrix PTR [eax])._m31,edx        ;42
        mov     (vmp3DMatrix PTR [eax])._m32,ecx        ;43

        ret
vmp_MatrixSetTranslateAsm       endp




;       MMX Version (Only use for 3DNow! Not worth it otherwise due to EMMS!

        public  vmp_MatrixSetTranslateAsmMMX
vmp_MatrixSetTranslateAsmMMX    proc    near
        .MMX

ASSERT_PTR4( DWORD PTR [esp+4] )

        mov     eax,[esp+4]
        mov     ecx,[esp+8]             ; fTx
        mov     edx,[esp+12]            ; fTy

        movq    mm2,qword ptr vec1000-4 ; 1.0 0.0    {1 0} 
        pxor    mm0,mm0                 ; 0.0 0.0    {0 0} 
        movd    mm1,vec1000             ; 0.0 1.0    {0 1} 

        movq    (qword ptr [eax+0 ]),mm1        ;00,01 = 1.0 0.0
        movq    (qword ptr [eax+08]),mm0        ;02,03 = 0.0 0.0

        movq    (qword ptr [eax+16]),mm2        ;10,11 = 0.0 1.0
        movq    (qword ptr [eax+24]),mm0        ;12,13 =         0.0 0.0
           
        movq    (qword ptr [eax+32]),mm0        ;20,21 = 0.0 0.0
        movq    (qword ptr [eax+40]),mm1        ;22,23 =         1.0 0.0

        mov     (vmp3DMatrix PTR [eax])._m30,ecx        ;Tx
        mov     ecx,[esp+16]            ; fTz
        mov     (vmp3DMatrix PTR [eax])._m31,edx        ;Ty

        mov     (vmp3DMatrix PTR [eax])._m33,FLOAT1     ;44 = 1.0
        mov     (vmp3DMatrix PTR [eax])._m32,ecx        ;43

        ret
vmp_MatrixSetTranslateAsmMMX    endp



;       SSE Version - Aligned

        public  vmp_MatrixSetTranslateAsmSSE
vmp_MatrixSetTranslateAsmSSE    proc    near
        .XMM

ASSERT_PTR16( DWORD PTR [esp+4] )
                                        ;      x y z w       w z y x
        movaps  xmm0,oword ptr vec1000  ; aMx:[1 0 0 0]    H[0 0 0 1]L
        mov     eax,[esp+4]
        mov     ecx,[esp+8]             ; fTx
        mov     edx,[esp+12]            ; fTy

if 01
        movaps  xmm1,xmm0
        movaps  xmm2,xmm0
        movaps  xmm3,xmm0
        shufps  xmm1,xmm0,11110011b     ;     [0 1 0 0]     [0 0 1 0]
        shufps  xmm2,xmm0,11001111b     ;     [0 0 1 0]     [0 1 0 0]
else
        movups  xmm1,oword ptr vec1000-4
        movups  xmm2,oword ptr vec1000-8
endif

        mov     (vmp3DMatrix PTR [eax])._m30,ecx        ;41
        mov     ecx,[esp+16]            ; fTz
        mov     (vmp3DMatrix PTR [eax])._m31,edx        ;42

        movaps  (oword ptr [eax+0 ]),xmm0
        movaps  (oword ptr [eax+16]),xmm1
        movaps  (oword ptr [eax+32]),xmm2

        mov     (vmp3DMatrix PTR [eax])._m33,FLOAT1     ;44 = 1.0
        mov     (vmp3DMatrix PTR [eax])._m32,ecx        ;43

        ret
vmp_MatrixSetTranslateAsmSSE    endp



;_________________________________________________________________________________________________
;=================================================================================================
;       Matrix - Set Scaling
;
;        | Sx 0  0  0 |
;        | 0  Sy 0  0 |
;        | 0  0  Sz 0 |
;        | 0  0  0  1 |
;
;void vmp_MatrixSetScaling( vmp3DMatrix Mx, float fSx, float fSy, float fSz )
;{
;                     Mx[0][1]=     Mx[0][2]=     Mx[0][3] = 
;       Mx[1][0]=                   Mx[1][2]=     Mx[1][3] = 
;       Mx[2][0]=     Mx[2][1]=                   Mx[2][3] =
;       Mx[3][0]=     Mx[3][1]=     Mx[3][2]=                0.0f;
;
;    Mx[0][0]=fSx; Mx[1][1]=fSy; Mx[2][2]=fSz; Mx[3][3] = 1.0f;
;}
;_________________________________________________________________________________________________

        public  vmp_MatrixSetScalingAsm
vmp_MatrixSetScalingAsm proc    near

ASSERT_PTR4( DWORD PTR [esp+4] )                ; dMx


        mov     ecx,[esp+8]                             ; fSx=
        mov     eax,[esp+4]                             ; dMx

        mov     edx,FLOAT0                              ; 0.0f

        mov     (vmp3DMatrix PTR [eax])._m00,ecx        ;[0] = fSx
        mov     ecx,[esp+12]                            ; fSy=
        mov     (vmp3DMatrix PTR [eax])._m01,edx        ;[1] = 0.0
        mov     (vmp3DMatrix PTR [eax])._m02,edx        ;[2]
        mov     (vmp3DMatrix PTR [eax])._m03,edx        ;[3]

        mov     (vmp3DMatrix PTR [eax])._m10,edx        ;[4]
        mov     (vmp3DMatrix PTR [eax])._m11,ecx        ;[5] = fSy
        mov     ecx,[esp+16]                            ; fSz=
        mov     (vmp3DMatrix PTR [eax])._m12,edx        ;[6]
        mov     (vmp3DMatrix PTR [eax])._m13,edx        ;[7]

        mov     (vmp3DMatrix PTR [eax])._m20,edx        ;[8]
        mov     (vmp3DMatrix PTR [eax])._m21,edx        ;[9]
        mov     (vmp3DMatrix PTR [eax])._m22,ecx        ;[10] = fSz
        mov     ecx,FLOAT1
        mov     (vmp3DMatrix PTR [eax])._m23,edx        ;[11]

        mov     (vmp3DMatrix PTR [eax])._m30,edx        ;[12]
        mov     (vmp3DMatrix PTR [eax])._m31,edx        ;[13]
        mov     (vmp3DMatrix PTR [eax])._m32,edx        ;[14]
        mov     (vmp3DMatrix PTR [eax])._m33,ecx        ;[15] = 1.0

        ret
vmp_MatrixSetScalingAsm endp




;       MMX Version (Only use for 3DNow! Not worth it otherwise due to EMMS!

        public  vmp_MatrixSetScalingAsmMMX
vmp_MatrixSetScalingAsmMMX      proc    near
        .MMX

ASSERT_PTR4( DWORD PTR [esp+4] )

        movq    mm0,qword ptr vec1000-8  ;[0 0 0 0]

        mov     eax,[esp+4]
        mov     ecx,[esp+8]                             ; fSx
        mov     edx,[esp+12]                            ; fSy

        movq    (oword ptr [eax+(1*4)]),mm0             ; [1,2]
        movq    (oword ptr [eax+(3*4)]),mm0             ; [3  4]

        movq    (oword ptr [eax+(6*4)]),mm0             ; [6,7]
        movq    (oword ptr [eax+(8*4)]),mm0             ; [8,9]

        movq    (oword ptr [eax+(11*4)]),mm0            ; [11,    12]
        movq    (oword ptr [eax+(13*4)]),mm0            ; [13,14]

        mov     (vmp3DMatrix PTR [eax])._m00,ecx        ;[0]=fSx
        mov     ecx,[esp+16]                            ;fSz
        mov     (vmp3DMatrix PTR [eax])._m11,edx        ;[5]=fSy
        mov     (vmp3DMatrix PTR [eax])._m33,FLOAT1     ;[15]= 1.0
        mov     (vmp3DMatrix PTR [eax])._m22,ecx        ;[10]=fSz

        ret
vmp_MatrixSetScalingAsmMMX      endp



;       SSE Version - Aligned

        public  vmp_MatrixSetScalingAsmSSE
vmp_MatrixSetScalingAsmSSE      proc    near
        .XMM

ASSERT_PTR16( DWORD PTR [esp+4] )

        movaps  xmm0,oword ptr vec1000-16  ;[0 0 0 0]

        mov     eax,[esp+4]
        mov     ecx,[esp+8]                             ; fSx
        mov     edx,[esp+12]                            ; fSy

        movups  (oword ptr [eax+(1*4)]),xmm0            ; [1,2,3  4]
        movups  (oword ptr [eax+(6*4)]),xmm0            ; [6,7    8,9]
        movups  (oword ptr [eax+(11*4)]),xmm0           ; [11,    12,13,14]

        mov     (vmp3DMatrix PTR [eax])._m00,ecx        ;[0]=fSx
        mov     ecx,[esp+16]                            ;fSz
        mov     (vmp3DMatrix PTR [eax])._m11,edx        ;[5]=fSy
        mov     (vmp3DMatrix PTR [eax])._m33,FLOAT1     ;[15]= 1.0
        mov     (vmp3DMatrix PTR [eax])._m22,ecx        ;[10]=fSz

        ret
vmp_MatrixSetScalingAsmSSE      endp



;_________________________________________________________________________________________________
;=================================================================================================
;       Matrix - Add  mD = mA + mB
;
;void vmp_MatrixAdd( vmp3DMatrix dMx, const vmp3DMatrix aMx, const vmp3DMatrix bMx )
;{
;       dMx[0][0]=aMx[0][0] + bMx[0][0];  dMx[0][1]=aMx[0][1] + bMx[0][1];
;       dMx[0][2]=aMx[0][2] + bMx[0][2];  dMx[0][3]=aMx[0][3] + bMx[0][3];
;
;       dMx[1][0]=aMx[1][0] + bMx[1][0];  dMx[1][1]=aMx[1][1] + bMx[1][1];
;       dMx[1][2]=aMx[1][2] + bMx[1][2];  dMx[1][3]=aMx[1][3] + bMx[1][3];
;
;       dMx[2][0]=aMx[2][0] + bMx[2][0];  dMx[2][1]=aMx[2][1] + bMx[2][1];
;       dMx[2][2]=aMx[2][2] + bMx[2][2];  dMx[2][3]=aMx[2][3] + bMx[2][3];
;
;       dMx[3][0]=aMx[3][0] + bMx[3][0];  dMx[3][1]=aMx[3][1] + bMx[3][1];
;       dMx[3][2]=aMx[3][2] + bMx[3][2];  dMx[3][3]=aMx[3][3] + bMx[3][3];
;}
;_________________________________________________________________________________________________

;       3DNow! Version - (UN)Aligned

        public  vmp_MatrixAddAsm3DNow
vmp_MatrixAddAsm3DNow   proc    near    ;;; , dMx:ptr, aMx:ptr, bMx:ptr
        .K3D

ASSERT_PTR4( DWORD PTR [esp+12] )               ; bMx
ASSERT_PTR4( DWORD PTR [esp+8] )                ; aMx
ASSERT_PTR4( DWORD PTR [esp+4] )                ; dMx

        mov     ecx,[esp+12]            ; bMx
        mov     eax,[esp+8]             ; aMx
        mov     edx,[esp+4]             ; dMx

        i = 0
REPT    4
        movq    mm0,[eax+0+i]           ; [A1 A0]
        movq    mm2,[ecx+0+i]           ; [B1 B0]
        movq    mm1,[eax+8+i]           ; [A3 A2]
        movq    mm3,[ecx+8+i]           ; [B3 B2]

        pfadd   mm0,mm2                 ; [A1+B1 A0+B0]
        pfadd   mm1,mm3                 ; [A3+B3 A2+B2]

        movq    [edx+0+i],mm0           ; [D1 D0]
        movq    [edx+8+i],mm1           ; [D3 D2]

        i = i + 16
ENDM
        ret
vmp_MatrixAddAsm3DNow   endp



;       SSE Version - (Aligned)

        public  vmp_MatrixAddAsmSSE
vmp_MatrixAddAsmSSE     proc    near    ;;; , dMx:ptr, aMx:ptr
        .XMM

ASSERT_PTR16( DWORD PTR [esp+12] )              ; bMx
ASSERT_PTR16( DWORD PTR [esp+8] )               ; aMx
ASSERT_PTR16( DWORD PTR [esp+4] )               ; dMx

        mov     ecx,[esp+12]            ; bMx
        mov     eax,[esp+8]             ; aMx
        mov     edx,[esp+4]             ; dMx

        i = 0
REPT    4
        movaps  xmm0,[eax+0+i]          ; [A3 A2 A1 A0]
        movaps  xmm1,[ecx+0+i]          ; [B3 B2 B1 B0]

        addps   xmm0,xmm1               ; [A3+B3 A2+B2 A1+B1 A0+B0]

        movaps  [edx+0+i],xmm0          ; [D3 D2 D1 D0]
        i = i + 16
ENDM
        ret
vmp_MatrixAddAsmSSE     endp




;_________________________________________________________________________________________________
;=================================================================================================
;void vmp_MatrixMul( vmp3DMatrix dMx, const vmp3DMatrix aMx, const vmp3DMatrix bMx )
;{
;       vmp3DMatrix mx;
;       uint u;
;       vmp3DQVector *pvA, *pvD;
;       float *pB;
;
;       pB = (float *)bMx;
;       pvA = (vmp3DQVector*)aMx;
;       pvD = (vmp3DQVector*)mx;
;
;       for ( u=0; u < 4; u++ )
;         {
;               pvD->x = (*(pB+0) * pvA->x) + (*(pB+4) * pvA->y) + (*(pB+8) * pvA->z) + (*(pB+12) * pvA->w);
;               pvD->y = (*(pB+1) * pvA->x) + (*(pB+5) * pvA->y) + (*(pB+9) * pvA->z) + (*(pB+13) * pvA->w);
;               pvD->z = (*(pB+2) * pvA->x) + (*(pB+6) * pvA->y) + (*(pB+10)* pvA->z) + (*(pB+14) * pvA->w);
;               pvD->w = (*(pB+3) * pvA->x) + (*(pB+7) * pvA->y) + (*(pB+11)* pvA->z) + (*(pB+15) * pvA->w);
;               pvA++;
;               pvD++;
;         }
;
;       vmp_MatrixCopyGeneric(dMx, mx);
;}
;_________________________________________________________________________________________________


;       3DNow! Version - (UN)Aligned

        public  vmp_MatrixMulAsm3DNow
vmp_MatrixMulAsm3DNow   proc    near
        .K3D

ASSERT_PTR16( DWORD PTR [esp+8] )       ; aMx
ASSERT_PTR16( DWORD PTR [esp+4] )       ; dMx
ASSERT_PTR16( DWORD PTR [esp+12] )      ; bMx

        mov     eax,[esp+8]             ; aMx
        mov     ecx,[esp+12]            ; bMx
        mov     edx,[esp+4]             ; dMx

        i = 0
REPT    4
                ; upper left quad (1st Row)

        movq    mm4,[eax+i+0]           ;               {a1 a0}
        movq    mm0,[ecx+(0*4)]         ; [0 1 X X]     {b1 b0}
        movq    mm1,[ecx+(2*4)]         ; [X X 2 3]     {b3 b2}

        movq    mm5,mm4                 ;               {a1 a0}
        punpckldq mm4,mm4                       ;               {a0 a0}

        movq    mm2,[ecx+(4*4)]         ; [4 5 X X]     {b5 b4}
        movq    mm3,[ecx+(6*4)]         ; [X X 6 7]     {b7 b6}

        punpckhdq mm5,mm5                       ;               {a1 a1}

        pfmul   mm0,mm4                 ;               {b1*a0 b0*a0}
        pfmul   mm1,mm4                 ;               {b3*a0 b2*a0}
        pfmul   mm2,mm5                 ;               {b5*a1 b4*a1}
        pfmul   mm3,mm5                 ;               {b7*a1 b6*a1}
        pfadd   mm0,mm2                 ;
        pfadd   mm1,mm3

                ; upper right quad (1st Row)

        movq    mm4,[eax+i+8]           ;               {a3 a2}
        movq    mm6,[ecx+(8*4)]         ; [8 9 X X]     {b9 b8}
        movq    mm7,[ecx+(10*4)]                ; [X X A B]     {bB bA}

        movq    mm5,mm4                 ;               {a3 a2}
        punpckldq mm4,mm4                       ;               {a2 a2}

        movq    mm2,[ecx+(12*4)]                ; [C D X X]     {bD bC}
        movq    mm3,[ecx+(14*4)]                ; [X X E E]     {bF bE}

        punpckhdq mm5,mm5                       ;               {a3 a3}

        pfmul   mm6,mm4                 ;               {b9*a2 b8*a2}
        pfmul   mm7,mm4                 ;               {bB*a2 bA*a2}
        pfmul   mm2,mm5                 ;               {bD*a3 bC*a3}
        pfmul   mm3,mm5                 ;               {bF*a3 bE*a3}
        pfadd   mm0,mm6                 ;
        pfadd   mm1,mm7
        pfadd   mm0,mm2
        pfadd   mm1,mm3
        
        movq    [edx+i+0],mm0           ; [0 1 X X]     {d1 d0}
        movq    [edx+i+8],mm1           ; [X X 2 3]     {d3 d2}
i = i + 16
ENDM
        ret
vmp_MatrixMulAsm3DNow   endp

;_________________________________________________________________________________________________


;       SSE Version - (Aligned)

        public  vmp_MatrixMulAsmSSE
vmp_MatrixMulAsmSSE     proc    near
        .XMM

ASSERT_PTR16( DWORD PTR [esp+8] )       ; aMx
ASSERT_PTR16( DWORD PTR [esp+4] )       ; dMx
ASSERT_PTR16( DWORD PTR [esp+12] )      ; bMx

        mov     eax,[esp+8]             ; aMx
        mov     ecx,[esp+12]            ; bMx
        mov     edx,[esp+4]             ; dMx

        movaps  xmm4,[ecx+(0*4)]        ; [b3 b2 b1 b0] wzyx
        movaps  xmm5,[ecx+(4*4)]        ; [b7 b6 b5 b4] wzyx
        movaps  xmm6,[ecx+(8*4)]        ; [bB bA b9 b8] wzyx
        movaps  xmm7,[ecx+(12*4)]       ; [bF bE bD bC] wzyx

if 01           ; Rolled Macro
        i = 0
        REPT    4
        movaps  xmm0,[eax+i]            ; [a3 a2 a1 a0] wzyx

        movaps  xmm1,xmm0
        movaps  xmm2,xmm0
        movaps  xmm3,xmm0
        shufps  xmm0,xmm0,00000000b     ; [a0 a0 a0 a0]
        shufps  xmm1,xmm1,01010101b     ; [a1 a1 a1 a1]
        shufps  xmm2,xmm2,10101010b     ; [a2 a2 a2 a2]
        shufps  xmm3,xmm3,11111111b     ; [a3 a3 a3 a3]

        mulps   xmm0,xmm4               ; [a0*b3 a0*b2 a0*b1 a0*b0]
        mulps   xmm1,xmm5               ; [a1*b7 a1*b6 a1*b5 a1*b4]
        mulps   xmm2,xmm6               ; [a2*bB a2*bA a2*b9 a2*b8]
        mulps   xmm3,xmm7               ; [a3*bF a3*bE a3*bD a3*bC]

        addps   xmm2,xmm0
        addps   xmm3,xmm1
        addps   xmm3,xmm2
        movaps  [edx+i],xmm3            ; [d3 d2 d1 d0] wzyx

        i = i + (4*4)
        ENDM    


else
        movaps  xmm0,[eax+(0*4)]        ; [a3 a2 a1 a0] wzyx

        movaps  xmm1,xmm0
        movaps  xmm2,xmm0
        movaps  xmm3,xmm0
        shufps  xmm0,xmm0,00000000b     ; [a0 a0 a0 a0]
        shufps  xmm1,xmm1,01010101b     ; [a1 a1 a1 a1]
        shufps  xmm2,xmm2,10101010b     ; [a2 a2 a2 a2]
        shufps  xmm3,xmm3,11111111b     ; [a3 a3 a3 a3]

        mulps   xmm0,xmm4               ; [a0*b3 a0*b2 a0*b1 a0*b0]
        mulps   xmm1,xmm5               ; [a1*b7 a1*b6 a1*b5 a1*b4]
        mulps   xmm2,xmm6               ; [a2*bB a2*bA a2*b9 a2*b8]
        mulps   xmm3,xmm7               ; [a3*bF a3*bE a3*bD a3*bC]

        addps   xmm2,xmm0
        addps   xmm3,xmm1
        addps   xmm3,xmm2
        movaps  [edx+(0*4)],xmm3        ; [d3 d2 d1 d0] wzyx
        
                ; 2nd block

        movaps  xmm0,[eax+(4*4)]        ; [a7 a6 a5 a4] wzyx

        movaps  xmm1,xmm0
        movaps  xmm2,xmm0
        movaps  xmm3,xmm0
        shufps  xmm0,xmm0,00000000b     ; [a4 a4 a4 a4]
        shufps  xmm1,xmm1,01010101b     ; [a5 a5 a5 a5]
        shufps  xmm2,xmm2,10101010b     ; [a6 a6 a6 a6]
        shufps  xmm3,xmm3,11111111b     ; [a7 a7 a7 a7]

        mulps   xmm0,xmm4               ; [a4*b3 a4*b2 a4*b1 a4*b0]
        mulps   xmm1,xmm5               ; [a5*b7 a5*b6 a5*b5 a5*b4]
        mulps   xmm2,xmm6               ; [a6*bB a6*bA a6*b9 a6*b8]
        mulps   xmm3,xmm7               ; [a7*bF a7*bE a7*bD a7*bC]

        addps   xmm2,xmm0
        addps   xmm3,xmm1
        addps   xmm3,xmm2
        movaps  [edx+(4*4)],xmm3        ; [d7 d6 d5 d4] wzyx

                ; 3rd block

        movaps  xmm0,[eax+(8*4)]        ; [aB aA a9 a8] wzyx

        movaps  xmm1,xmm0
        movaps  xmm2,xmm0
        movaps  xmm3,xmm0
        shufps  xmm0,xmm0,00000000b     ; [a8 a8 a8 a8]
        shufps  xmm1,xmm1,01010101b     ; [a9 a9 a9 a9]
        shufps  xmm2,xmm2,10101010b     ; [aA aA aA aA]
        shufps  xmm3,xmm3,11111111b     ; [aB aB aB aB]

        mulps   xmm0,xmm4               ; [a8*b3 a8*b2 a8*b1 a8*b0]
        mulps   xmm1,xmm5               ; [a9*b7 a9*b6 a9*b5 a9*b4]
        mulps   xmm2,xmm6               ; [aA*bB aA*bA aA*b9 aA*b8]
        mulps   xmm3,xmm7               ; [aB*bF aB*bE aB*bD aB*bC]

        addps   xmm2,xmm0
        addps   xmm3,xmm1
        addps   xmm3,xmm2
        movaps  [edx+(8*4)],xmm3        ; [dB dA d9 d8]
        
                ; 4th block

        movaps  xmm0,[eax+(12*4)]       ; [aF aE aD aC] wzyx

        movaps  xmm1,xmm0
        movaps  xmm2,xmm0
        movaps  xmm3,xmm0
        shufps  xmm0,xmm0,00000000b     ; [aC aC aC aC]
        shufps  xmm1,xmm1,01010101b     ; [aD aD aD aD]
        shufps  xmm2,xmm2,10101010b     ; [aE aE aE aE]
        shufps  xmm3,xmm3,11111111b     ; [aF aF aF aF]

        mulps   xmm0,xmm4               ; [aC*b3 aC*b2 aC*b1 aC*b0]
        mulps   xmm1,xmm5               ; [aD*b7 aD*b6 aD*b5 aD*b4]
        mulps   xmm2,xmm6               ; [aE*bB aE*bA aE*b9 aE*b8]
        mulps   xmm3,xmm7               ; [aF*bF aF*bE aF*bD aF*bC]

        addps   xmm2,xmm0
        addps   xmm3,xmm1
        addps   xmm3,xmm2
        movaps  [edx+(12*4)],xmm3       ; [dF dE dD dC]
endif

        ret
vmp_MatrixMulAsmSSE     endp




Matrix3DX86     ends
        end

