
//
//  vecBug.cpp - VMP 3D Debug Module
//
//  This document contains file that you may use at your own risk in your own programs.
//
//  This module is designed to work with the exercises for this book.
//
//      Hopefully by now you have realized the necessity of 16 byte aligned data
//      and so this program is written as such!
//
//  Created - 5 Oct 2001 - James Leiterman
//
//  Last Modification
//
//
//  stab=4
//

#include "KariType.h"                                   // Master Definitions

    //  3rd Party Libraries
    
#include    <assert.h>
#include    <stdio.h>

    //  Book Development Library Modules

#include "CpuAsm.h"                                     // CPU Module
#include "vmp3D.h"
#include "vecBug.h"



//
//  Memory Dump
//
void vmp_LogMem( const void * const vp, uint size )
{
    uint  x, y, run, tail, col, n;
    char *p, buf[256];
    byte *mp;

    ASSERT_PTR(vp);
    ASSERT_ZERO(size);

    mp = (byte *) vp;

    run = 16;
    col = tail = 0;

        //  For all lines

    for ( y = 0; y < size;  )
      {
        if ( size-y < run )
          {
            run = size-y;                               // Display only that requested!
            n = 16 - run;                               // trailing places
            tail = n * 3;                               // trailing spaces
            tail += ((n+3) >> 2);                       // column separators
          }

        p = buf + sprintf(buf, "%8.8x ", mp );

            //  Hex values

        for ( x = 0; x < run; x++ )
          {
            p += sprintf( p, " %2.2x", *(mp+x) );

            if (( ++col % 4 ) == 0 )
              {
                *p++ = ' ';
              }
          }

            //  Any Trailing spaces ?

        while ( tail-- )
          {
            *p++ = ' ';
          }

            //  ASCII chars

        for ( x = 0; x < run; x++ )
          {
            if (( 0x20 <= *mp ) && ( *mp <= 0x7f ))
              {
                *p++ = *mp++;
              }
            else
              {
                *p++ = '.';
                mp++;
              }
          }

        *p++ = '\r';
        *p++ = '\n';
        *p = 0;
        printf( buf );

        y += run;
        run = 16;
        tail = col = 0;
      }

    printf( "\r\n" );
}



//
//  Print Dump a list of single-precision floats (4 per line)
//

void vmp_FloatPrint( const float * pf, uint nCnt )
{
    int i;

    ASSERT_PTR4(pf);
    ASSERT_ZERO(nCnt);

    vmp_SIMDEntry();

    do  {
        i = (--nCnt >= 4) ? 3 : (nCnt & 3);
        nCnt -= i;
        
        printf( "%8.8x ", pf );

        do {
            printf( "%lf ", (double)(*pf) );
            pf++;
          } while (i-- > 0);

        printf( "\n" );
      } while (nCnt);

    vmp_SIMDExit();
}



//
//  Print Vector
//
void vmp_VecPrint( const vmp3DVector *pVec )
{
    vmp_FloatPrint( (float *)pVec, 3 );
}



//
//  Print Quad Vector
//
void vmp_QVecPrint( const vmp3DQVector *pVec )
{
    vmp_FloatPrint( (float *)pVec, 4 );
}



//
//  Print Quaternion Vector
//
void vmp_QuatPrint( const vmpQuat *pVec )
{
    vmp_FloatPrint( (float *)pVec, 4 );
}



//
//  Print Dump of a Matrix
//

void vmp_MatrixPrint( const vmp3DMatrix Mx )
{
    ASSERT_PTR4(Mx);

    vmp_SIMDEntry();

    printf( "%8.8x ", Mx );
    printf( "%lf %lf %lf %lf\n",   (double)Mx[0][0], (double)Mx[0][1], (double)Mx[0][2], (double)Mx[0][3]);
    printf( "         %lf %lf %lf %lf\n",   (double)Mx[1][0], (double)Mx[1][1], (double)Mx[1][2], (double)Mx[1][3]);
    printf( "         %lf %lf %lf %lf\n",   (double)Mx[2][0], (double)Mx[2][1], (double)Mx[2][2], (double)Mx[2][3]);
    printf( "         %lf %lf %lf %lf\n\n", (double)Mx[3][0], (double)Mx[3][1], (double)Mx[3][2], (double)Mx[3][3]);

    vmp_SIMDExit();
}



//
//___________________________________________________________________________
//  *** TEST JIGS ***

//
//  Splat a SPFP value into all sixteen elements
//

void vmp_MatrixSplat( vmp3DMatrix Mx, float f )
{
    uint n;
    float *pF;

    pF = (float *)Mx;

    n = 16;
    while(n--)
      {
        *pF = f;
        pF++;
      }
}



//
//  Fill Matrix with floats
//
void vmp_MatrixTestFill( vmp3DMatrix Mx )
{
    uint n;
    float *pF, f;

    pF = (float *)Mx;

    f = 0.0f;

    n = 16;
    while(n--)
      {
        *pF = f;
        f += 1.0f;
        pF++;
      }
}



